@@ -1146,9 +1146,9 @@ convert_from_openpgp_native (ctrl_t ctrl,
 
           if (!agent_protect (*r_key, passphrase,
                               &protectedkey, &protectedkeylen,
-                              ctrl->s2k_count, -1))
+                              ctrl->s2k_count))
             agent_write_private_key (grip, protectedkey, protectedkeylen, 1,
-                                     NULL, NULL, 0);
+                                     NULL, NULL, NULL, 0);
           xfree (protectedkey);
         }
       else
@@ -1157,7 +1157,7 @@ convert_from_openpgp_native (ctrl_t ctrl,
           agent_write_private_key (grip,
                                    *r_key,
                                    gcry_sexp_canon_len (*r_key, 0, NULL,NULL),
-                                   1, NULL, NULL, 0);
+                                   1, NULL, NULL, NULL, 0);
         }
     }
 
