@@ -39,6 +39,12 @@
 #define O_BINARY 0
 #endif
 
+
+static gpg_error_t read_key_file (const unsigned char *grip,
+                                  gcry_sexp_t *result, nvc_t *r_keymeta);
+static gpg_error_t is_shadowed_key (gcry_sexp_t s_skey);
+
+
 /* Helper to pass data to the check callback of the unprotect function. */
 struct try_unprotect_arg_s
 {
@@ -50,6 +56,22 @@ struct try_unprotect_arg_s
 };
 
 
+/* Return the file name for the 20 byte keygrip GRIP.  With FOR_NEW
+ * create a file name for later renaming to the actual name.  Return
+ * NULL on error. */
+static char *
+fname_from_keygrip (const unsigned char *grip, int for_new)
+{
+  char hexgrip[40+4+4+1];
+
+  bin2hex (grip, 20, hexgrip);
+  strcpy (hexgrip+40, for_new? ".key.tmp" : ".key");
+
+  return make_filename_try (gnupg_homedir (), GNUPG_PRIVATE_KEYS_DIR,
+                            hexgrip, NULL);
+}
+
+
 /* Replace all linefeeds in STRING by "%0A" and return a new malloced
  * string.  May return NULL on memory error.  */
 static char *
@@ -84,124 +106,50 @@ linefeed_to_percent0A (const char *string)
  * storage.  With FORCE passed as true an existing key with the given
  * GRIP will get overwritten.  If SERIALNO and KEYREF are given a
  * Token line is added to the key if the extended format is used.  If
- * TIMESTAMP is not zero and the key doies not yet exists it will be
+ * TIMESTAMP is not zero and the key does not yet exists it will be
  * recorded as creation date.  */
-int
+gpg_error_t
 agent_write_private_key (const unsigned char *grip,
                          const void *buffer, size_t length, int force,
                          const char *serialno, const char *keyref,
+                         const char *dispserialno,
                          time_t timestamp)
 {
   gpg_error_t err;
-  char *fname;
+  char *fname = NULL;
+  char *tmpfname = NULL;
   estream_t fp;
-  char hexgrip[40+4+1];
-  int update, newkey;
+  int newkey;
   nvc_t pk = NULL;
   gcry_sexp_t key = NULL;
-  int remove = 0;
+  int removetmp = 0;
+  char *token0 = NULL;
   char *token = NULL;
+  char *dispserialno_buffer = NULL;
+  char **tokenfields = NULL;
+  int is_regular;
+  int blocksigs = 0;
 
-  bin2hex (grip, 20, hexgrip);
-  strcpy (hexgrip+40, ".key");
+  fname = fname_from_keygrip (grip, 0);
+  if (!fname)
+    return gpg_error_from_syserror ();
 
-  fname = make_filename (gnupg_homedir (), GNUPG_PRIVATE_KEYS_DIR,
-                         hexgrip, NULL);
-
-  /* FIXME: Write to a temp file first so that write failures during
-     key updates won't lead to a key loss.  */
-
-  if (!force && !gnupg_access (fname, F_OK))
+  err = read_key_file (grip, &key, &pk);
+  if (err)
     {
-      log_error ("secret key file '%s' already exists\n", fname);
-      xfree (fname);
-      return gpg_error (GPG_ERR_EEXIST);
-    }
-
-  fp = es_fopen (fname, force? "rb+,mode=-rw" : "wbx,mode=-rw");
-  if (!fp)
-    {
-      gpg_error_t tmperr = gpg_error_from_syserror ();
-
-      if (force && gpg_err_code (tmperr) == GPG_ERR_ENOENT)
-        {
-          fp = es_fopen (fname, "wbx,mode=-rw");
-          if (!fp)
-            tmperr = gpg_error_from_syserror ();
-        }
-      if (!fp)
-        {
-          log_error ("can't create '%s': %s\n", fname, gpg_strerror (tmperr));
-          xfree (fname);
-          return tmperr;
-        }
-      update = 0;
-      newkey = 1;
-    }
-  else if (force)
-    {
-      gpg_error_t rc;
-      char first;
-
-      /* See if an existing key is in extended format.  */
-      if (es_fread (&first, 1, 1, fp) != 1)
-        {
-          rc = gpg_error_from_syserror ();
-          log_error ("error reading first byte from '%s': %s\n",
-                     fname, strerror (errno));
-          xfree (fname);
-          es_fclose (fp);
-          return rc;
-        }
-
-      rc = es_fseek (fp, 0, SEEK_SET);
-      if (rc)
-        {
-          log_error ("error seeking in '%s': %s\n", fname, strerror (errno));
-          xfree (fname);
-          es_fclose (fp);
-          return rc;
-        }
-
-      if (first == '(')
-        {
-          /* Key is still in the old format - force it into extended
-           * format.  We do not request an update here because an
-           * existing key is not yet in extended key format and no
-           * extended infos are yet available.  */
-          update = 0;
-          newkey = 0;
-        }
+      if (gpg_err_code (err) == GPG_ERR_ENOENT)
+        newkey = 1;
       else
         {
-          /* Key is already in the extended format.  */
-          update = 1;
-          newkey = 0;
-        }
-    }
-  else
-    {
-      /* The key file did not exist: we assume this is a new key and
-       * write the Created: entry.  */
-      update = 0;
-      newkey = 1;
-    }
-
-
-  if (update)
-    {
-      int line;
-
-      err = nvc_parse_private_key (&pk, &line, fp);
-      if (err && gpg_err_code (err) != GPG_ERR_ENOENT)
-        {
-          log_error ("error parsing '%s' line %d: %s\n",
-                     fname, line, gpg_strerror (err));
+          log_error ("can't open '%s': %s\n", fname, gpg_strerror (err));
           goto leave;
         }
     }
-  else
+
+  if (!pk)
     {
+      /* Key is still in the old format or does not exist - create a
+       * new container.  */
       pk = nvc_new_private_key ();
       if (!pk)
         {
@@ -209,10 +157,13 @@ agent_write_private_key (const unsigned char *grip,
           goto leave;
         }
     }
-  es_clearerr (fp);
+
+  /* Check whether we already have a regular key.  */
+  is_regular = (key && gpg_err_code (is_shadowed_key (key)) != GPG_ERR_TRUE);
 
   /* Turn (BUFFER,LENGTH) into a gcrypt s-expression and set it into
    * our name value container.  */
+  gcry_sexp_release (key);
   err = gcry_sexp_sscan (&key, NULL, buffer, length);
   if (err)
     goto leave;
@@ -220,24 +171,56 @@ agent_write_private_key (const unsigned char *grip,
   if (err)
     goto leave;
 
+  /* Check that we do not update a regular key with a shadow key.  */
+  if (is_regular && gpg_err_code (is_shadowed_key (key)) == GPG_ERR_TRUE)
+    {
+      log_info ("updating regular key file '%s'"
+                " by a shadow key inhibited\n", fname);
+      err = 0;  /* Simply ignore the error.  */
+      goto leave;
+    }
+
+  /* Check that we update a regular key only in force mode.  */
+  if (is_regular && !force)
+    {
+      log_error ("secret key file '%s' already exists\n", fname);
+      err = gpg_error (GPG_ERR_EEXIST);
+      goto leave;
+    }
+
   /* If requested write a Token line.  */
   if (serialno && keyref)
     {
       nve_t item;
       const char *s;
+      size_t token0len;
 
-      token = strconcat (serialno, " ", keyref, NULL);
-      if (!token)
+      if (dispserialno)
+        {
+          /* Escape the DISPSERIALNO.  */
+          dispserialno_buffer = percent_plus_escape (dispserialno);
+          if (!dispserialno_buffer)
+            {
+              err = gpg_error_from_syserror ();
+              goto leave;
+            }
+          dispserialno = dispserialno_buffer;
+        }
+
+      token0 = strconcat (serialno, " ", keyref, NULL);
+      if (token0)
+        token = strconcat (token0, " - ", dispserialno? dispserialno:"-", NULL);
+      if (!token0 || !token)
         {
           err = gpg_error_from_syserror ();
           goto leave;
         }
 
-      /* fixme: the strcmp should compare only the first two strings.  */
+      token0len = strlen (token0);
       for (item = nvc_lookup (pk, "Token:");
            item;
            item = nve_next_value (item, "Token:"))
-        if ((s = nve_value (item)) && !strcmp (s, token))
+        if ((s = nve_value (item)) && !strncmp (s, token0, token0len))
           break;
       if (!item)
         {
@@ -248,6 +231,23 @@ agent_write_private_key (const unsigned char *grip,
           if (err)
             goto leave;
         }
+      else
+        {
+          /* Token exists: Update the display s/n.  It may have
+           * changed due to changes in a newer software version.  */
+          if (s && (tokenfields = strtokenize (s, " \t\n"))
+              && tokenfields[0] && tokenfields[1] && tokenfields[2]
+              && tokenfields[3]
+              && !strcmp (tokenfields[3], dispserialno))
+            ; /* No need to update Token entry.  */
+          else
+            {
+              err = nve_set (item, token);
+              if (err)
+                goto leave;
+            }
+        }
+
     }
 
   /* If a timestamp has been supplied and the key is new, write a
@@ -263,49 +263,61 @@ agent_write_private_key (const unsigned char *grip,
         goto leave;
     }
 
-  /* Back to start and write.  */
-  err = es_fseek (fp, 0, SEEK_SET);
-  if (err)
-    goto leave;
-
-  err = nvc_write (pk, fp);
-  if (!err)
-    err = es_fflush (fp);
-  if (err)
+  /* Create a temporary file for writing.  */
+  tmpfname = fname_from_keygrip (grip, 1);
+  fp = tmpfname ? es_fopen (tmpfname, "wbx,mode=-rw") : NULL;
+  if (!fp)
     {
-      log_error ("error writing '%s': %s\n", fname, gpg_strerror (err));
-      remove = 1;
+      err = gpg_error_from_syserror ();
+      log_error ("can't create '%s': %s\n", tmpfname, gpg_strerror (err));
       goto leave;
     }
 
-  if (ftruncate (es_fileno (fp), es_ftello (fp)))
+  err = nvc_write (pk, fp);
+  if (!err && es_fflush (fp))
+    err = gpg_error_from_syserror ();
+  if (err)
     {
-      err = gpg_error_from_syserror ();
-      log_error ("error truncating '%s': %s\n", fname, gpg_strerror (err));
-      remove = 1;
+      log_error ("error writing '%s': %s\n", tmpfname, gpg_strerror (err));
+      removetmp = 1;
       goto leave;
     }
 
   if (es_fclose (fp))
     {
       err = gpg_error_from_syserror ();
-      log_error ("error closing '%s': %s\n", fname, gpg_strerror (err));
-      remove = 1;
+      log_error ("error closing '%s': %s\n", tmpfname, gpg_strerror (err));
+      removetmp = 1;
       goto leave;
     }
   else
     fp = NULL;
 
+  err = gnupg_rename_file (tmpfname, fname, &blocksigs);
+  if (err)
+    {
+      err = gpg_error_from_syserror ();
+      log_error ("error renaming '%s': %s\n", tmpfname, gpg_strerror (err));
+      removetmp = 1;
+      goto leave;
+    }
+
   bump_key_eventcounter ();
 
  leave:
+  if (blocksigs)
+    gnupg_unblock_all_signals ();
   es_fclose (fp);
-  if (remove)
-    gnupg_remove (fname);
+  if (removetmp && tmpfname)
+    gnupg_remove (tmpfname);
   xfree (fname);
+  xfree (tmpfname);
+  xfree (token);
+  xfree (token0);
+  xfree (dispserialno_buffer);
+  xfree (tokenfields);
   gcry_sexp_release (key);
   nvc_release (pk);
-  xfree (token);
   return err;
 }
 
@@ -313,53 +325,63 @@ agent_write_private_key (const unsigned char *grip,
 gpg_error_t
 agent_update_private_key (const unsigned char *grip, nvc_t pk)
 {
-  char *fname, *fname0;
-  estream_t fp;
-  char hexgrip[40+8+1];
   gpg_error_t err;
+  char *fname0 = NULL;   /* The existing file name.  */
+  char *fname = NULL;    /* The temporary new file name.  */
+  estream_t fp = NULL;
+  int removetmp = 0;
+  int blocksigs = 0;
 
-  bin2hex (grip, 20, hexgrip);
-  strcpy (hexgrip+40, ".key.tmp");
-
-  fname = make_filename (gnupg_homedir (), GNUPG_PRIVATE_KEYS_DIR,
-                         hexgrip, NULL);
-  fname0 = xstrdup (fname);
-  if (!fname0)
+  fname0 = fname_from_keygrip (grip, 0);
+  if (!fname)
     {
       err = gpg_error_from_syserror ();
-      xfree (fname);
-      return err;
+      goto leave;
+    }
+  fname = fname_from_keygrip (grip, 1);
+  if (!fname)
+    {
+      err = gpg_error_from_syserror ();
+      goto leave;
     }
-  fname0[strlen (fname)-4] = 0;
 
   fp = es_fopen (fname, "wbx,mode=-rw");
   if (!fp)
     {
       err = gpg_error_from_syserror ();
-
       log_error ("can't create '%s': %s\n", fname, gpg_strerror (err));
-      xfree (fname);
-      return err;
+      goto leave;
     }
 
   err = nvc_write (pk, fp);
   if (err)
-    log_error ("error writing '%s': %s\n", fname, gpg_strerror (err));
-
-  es_fclose (fp);
-
-#ifdef HAVE_W32_SYSTEM
-  /* No atomic mv on W32 systems.  */
-  gnupg_remove (fname0);
-#endif
-  if (rename (fname, fname0))
     {
-      err = gpg_error_from_errno (errno);
-      log_error (_("error renaming '%s' to '%s': %s\n"),
-                 fname, fname0, strerror (errno));
+      log_error ("error writing '%s': %s\n", fname, gpg_strerror (err));
+      removetmp = 1;
+      goto leave;
     }
 
+  es_fclose (fp);
+  fp = NULL;
+
+  err = gnupg_rename_file (fname, fname0, &blocksigs);
+  if (err)
+    {
+      err = gpg_error_from_syserror ();
+      log_error ("error renaming '%s': %s\n", fname, gpg_strerror (err));
+      removetmp = 1;
+      goto leave;
+    }
+
+
+ leave:
+  if (blocksigs)
+    gnupg_unblock_all_signals ();
+  es_fclose (fp);
+  if (removetmp && fname)
+    gnupg_remove (fname);
   xfree (fname);
+  xfree (fname0);
   return err;
 }
 
@@ -832,7 +854,7 @@ unprotect (ctrl_t ctrl, const char *cache_nonce, const char *desc_text,
 
 /* Read the key identified by GRIP from the private key directory and
  * return it as an gcrypt S-expression object in RESULT.  If R_KEYMETA
- * is not NULl and the extended key format is used, the meta data
+ * is not NULL and the extended key format is used, the meta data
  * items are stored there.  However the "Key:" item is removed from
  * it.  On failure returns an error code and stores NULL at RESULT and
  * R_KEYMETA. */
@@ -846,18 +868,17 @@ read_key_file (const unsigned char *grip, gcry_sexp_t *result, nvc_t *r_keymeta)
   unsigned char *buf;
   size_t buflen, erroff;
   gcry_sexp_t s_skey;
-  char hexgrip[40+4+1];
   char first;
 
   *result = NULL;
   if (r_keymeta)
     *r_keymeta = NULL;
 
-  bin2hex (grip, 20, hexgrip);
-  strcpy (hexgrip+40, ".key");
-
-  fname = make_filename (gnupg_homedir (), GNUPG_PRIVATE_KEYS_DIR,
-                         hexgrip, NULL);
+  fname = fname_from_keygrip (grip, 0);
+  if (!fname)
+    {
+      return gpg_error_from_syserror ();
+    }
   fp = es_fopen (fname, "rb");
   if (!fp)
     {
@@ -973,12 +994,12 @@ remove_key_file (const unsigned char *grip)
 {
   gpg_error_t err = 0;
   char *fname;
-  char hexgrip[40+4+1];
 
-  bin2hex (grip, 20, hexgrip);
-  strcpy (hexgrip+40, ".key");
-  fname = make_filename (gnupg_homedir (), GNUPG_PRIVATE_KEYS_DIR,
-                         hexgrip, NULL);
+  fname = fname_from_keygrip (grip, 0);
+  if (!fname)
+    {
+      return gpg_error_from_syserror ();
+    }
   if (gnupg_remove (fname))
     err = gpg_error_from_syserror ();
   xfree (fname);
@@ -1377,6 +1398,29 @@ agent_key_from_file (ctrl_t ctrl, const char *cache_nonce,
 }
 
 
+/* This function returns GPG_ERR_TRUE if S_SKEY represents a shadowed
+ * key.  0 is return for other key types.  Any other error may occur
+ * if S_SKEY is not valid.  */
+static gpg_error_t
+is_shadowed_key (gcry_sexp_t s_skey)
+{
+  gpg_error_t err;
+  unsigned char *buf;
+  size_t buflen;
+
+  err = make_canon_sexp (s_skey, &buf, &buflen);
+  if (err)
+    return err;
+
+  if (agent_private_key_type (buf) == PRIVATE_KEY_SHADOWED)
+    err = gpg_error (GPG_ERR_TRUE);
+
+  wipememory (buf, buflen);
+  xfree (buf);
+  return err;
+}
+
+
 /* Return the key for the keygrip GRIP.  The result is stored at
    RESULT.  This function extracts the key from the private key
    database and returns it as an S-expression object as it is.  On
@@ -1794,7 +1838,8 @@ agent_delete_key (ctrl_t ctrl, const char *desc_text,
 gpg_error_t
 agent_write_shadow_key (const unsigned char *grip,
                         const char *serialno, const char *keyid,
-                        const unsigned char *pkbuf, int force)
+                        const unsigned char *pkbuf, int force,
+                        const char *dispserialno)
 {
   gpg_error_t err;
   unsigned char *shadow_info;
@@ -1821,7 +1866,8 @@ agent_write_shadow_key (const unsigned char *grip,
     }
 
   len = gcry_sexp_canon_len (shdkey, 0, NULL, NULL);
-  err = agent_write_private_key (grip, shdkey, len, force, serialno, keyid, 0);
+  err = agent_write_private_key (grip, shdkey, len, force,
+                                 serialno, keyid, dispserialno, 0);
   xfree (shdkey);
   if (err)
     log_error ("error writing key: %s\n", gpg_strerror (err));
