@@ -497,6 +497,30 @@ nvc_set (nvc_t pk, const char *name, const char *value)
 }
 
 
+/* Update entry E to VALUE.  */
+gpg_error_t
+nve_set (nve_t e, const char *value)
+{
+  char *v;
+
+  if (!e)
+    return GPG_ERR_INV_ARG;
+
+  v = xtrystrdup (value? value:"");
+  if (!v)
+    return my_error_from_syserror ();
+
+  free_strlist_wipe (e->raw_value);
+  e->raw_value = NULL;
+  if (e->value)
+    wipememory (e->value, strlen (e->value));
+  xfree (e->value);
+  e->value = v;
+
+  return 0;
+}
+
+
 /* Delete the given entry from PK.  */
 void
 nvc_delete (nvc_t pk, nve_t entry)
@@ -592,7 +616,7 @@ nve_next_value (nve_t entry, const char *name)
 
 /* Return the string for the first entry in NVC with NAME.  If an
  * entry with NAME is missing in NVC or its value is the empty string
- * NULL is returned.  Note that the The returned string is a pointer
+ * NULL is returned.  Note that the the returned string is a pointer
  * into NVC.  */
 const char *
 nvc_get_string (nvc_t nvc, const char *name)
@@ -608,13 +632,14 @@ nvc_get_string (nvc_t nvc, const char *name)
 }
 
 
-/* Return true if NAME exists and its value is true; that is either
- * "yes", "true", or a decimal value unequal to 0.  */
+/* Return true (ie. a non-zero value) if NAME exists and its value is
+ * true; that is either "yes", "true", or a decimal value unequal to 0.  */
 int
 nvc_get_boolean (nvc_t nvc, const char *name)
 {
   nve_t item;
   const char *s;
+  int n;
 
   if (!nvc)
     return 0;
@@ -622,9 +647,12 @@ nvc_get_boolean (nvc_t nvc, const char *name)
   if (!item)
     return 0;
   s = nve_value (item);
-  if (s && (atoi (s)
-            || !ascii_strcasecmp (s, "yes")
-            || !ascii_strcasecmp (s, "true")))
+  if (!s)
+    return 0;
+  n = atoi (s);
+  if (n)
+    return n;
+  if (!ascii_strcasecmp (s, "yes") || !ascii_strcasecmp (s, "true"))
     return 1;
   return 0;
 }
