@@ -141,9 +141,12 @@ char *try_percent_escape (const char *str, const char *extra);
    NULL.  Returns a malloced buffer with the new string or NULL on a
    malloc error or if too many arguments are given.  */
 char *strconcat (const char *s1, ...) GPGRT_ATTR_SENTINEL(0);
+/* Same but taking a va_list.  */
+char *vstrconcat (const char *s1, va_list arg_ptr);
 /* Ditto, but die on error.  */
 char *xstrconcat (const char *s1, ...) GPGRT_ATTR_SENTINEL(0);
 
+
 char **strsplit (char *string, char delim, char replacement, int *count);
 
 /* Tokenize STRING using the set of delimiters in DELIM.  */
@@ -166,11 +169,16 @@ int compare_version_strings (const char *my_version, const char *req_version);
 /* Format a string so that it fits within about TARGET_COLS columns.  */
 char *format_text (const char *text, int target_cols, int max_cols);
 
-/* Substitute environmen variabales in STRING.  */
+/* Substitute variables in STRING.  */
+char *substitute_vars (const char *string,
+                       const char *(*getval)(void *cookie, const char *name),
+                       void *cookie);
 char *substitute_envvars (const char *string);
 
 
 /*-- mapstrings.c --*/
 const char *map_static_macro_string (const char *string);
+const char *map_static_strings (const char *domain, int key1, int key2,
+                                const char *string1, ...);
 
 #endif /*GNUPG_COMMON_STRINGHELP_H*/
