@@ -50,11 +50,6 @@
 #endif
 
 
-/* Bitflags to convey hints on what kind of signayire is created.  */
-#define SIGNHINT_KEYSIG  1
-#define SIGNHINT_SELFSIG 2
-
-
 /* Hack */
 static int recipient_digest_algo;
 
@@ -416,7 +411,10 @@ do_sign (ctrl_t ctrl, PKT_public_key *pksk, PKT_signature *sig,
   byte *dp;
   char *hexgrip;
 
-  if (pksk->timestamp > sig->timestamp )
+  /* An ADSK key commonly has a creation date older than the primary
+   * key.  For example because the ADSK is used as an archive key for
+   * a group of users.  */
+  if (pksk->timestamp > sig->timestamp && !(signhints & SIGNHINT_ADSK))
     {
       ulong d = pksk->timestamp - sig->timestamp;
       log_info (ngettext("key %s was created %lu second"
@@ -825,11 +823,10 @@ write_plaintext_packet (iobuf_t out, iobuf_t inp,
   /* Try to calculate the length of the data. */
   if ( !iobuf_is_pipe_filename (fname) && *fname)
     {
-      off_t tmpsize;
-      int overflow;
+      uint64_t tmpsize;
 
-      if (!(tmpsize = iobuf_get_filelength (inp, &overflow))
-          && !overflow && opt.verbose)
+      tmpsize = iobuf_get_filelength (inp);
+      if (!tmpsize && opt.verbose)
         log_info (_("WARNING: '%s' is an empty file\n"), fname);
 
       /* We can't encode the length of very large files because
@@ -964,7 +961,7 @@ write_signature_packets (ctrl_t ctrl,
       if (gcry_md_copy (&md, hash))
         BUG ();
 
-      build_sig_subpkt_from_sig (sig, pk);
+      build_sig_subpkt_from_sig (sig, pk, 0);
       mk_notation_policy_etc (ctrl, sig, NULL, pk);
       if (opt.flags.include_key_block && IS_SIG (sig))
         err = mk_sig_subpkt_key_block (ctrl, sig, pk);
@@ -1037,6 +1034,9 @@ sign_file (ctrl_t ctrl, strlist_t filenames, int detached, strlist_t locusr,
   int multifile = 0;
   u32 duration=0;
   pt_extra_hash_data_t extrahash = NULL;
+  char peekbuf[32];
+  int  peekbuflen = 0;
+
 
   pfx = new_progress_context ();
   afx = new_armor_context ();
@@ -1095,6 +1095,14 @@ sign_file (ctrl_t ctrl, strlist_t filenames, int detached, strlist_t locusr,
           goto leave;
 	}
 
+      peekbuflen = iobuf_ioctl (inp, IOBUF_IOCTL_PEEK, sizeof peekbuf, peekbuf);
+      if (peekbuflen < 0)
+        {
+          peekbuflen = 0;
+          if (DBG_FILTER)
+            log_debug ("peeking at input failed\n");
+        }
+
       handle_progress (pfx, inp, fname);
     }
 
@@ -1251,8 +1259,14 @@ sign_file (ctrl_t ctrl, strlist_t filenames, int detached, strlist_t locusr,
     {
       int compr_algo = opt.compress_algo;
 
-      /* If not forced by user */
-      if (compr_algo==-1)
+      if (!opt.explicit_compress_option
+          && is_file_compressed (peekbuf, peekbuflen))
+        {
+          if (opt.verbose)
+            log_info(_("'%s' already compressed\n"), fname? fname: "[stdin]");
+          compr_algo = 0;
+        }
+      else if (compr_algo==-1)
         {
           /* If we're not encrypting, then select_algo_from_prefs
            * will fail and we'll end up with the default.  If we are
@@ -1741,14 +1755,14 @@ sign_symencrypt_file (ctrl_t ctrl, const char *fname, strlist_t locusr)
  *
  * SIGCLASS is the type of signature to create.
  *
- * DIGEST_ALGO is the digest algorithm.  If it is 0 the function
- * selects an appropriate one.
- *
  * TIMESTAMP is the timestamp to use for the signature. 0 means "now"
  *
  * DURATION is the amount of time (in seconds) until the signature
  * expires.
  *
+ * If CACHED_NONCE is not NULL the agent may use it to avoid
+ * additional pinnetry popups for the same keyblock.
+ *
  * This function creates the following subpackets: issuer, created,
  * and expire (if duration is not 0).  Additional subpackets can be
  * added using MKSUBPKT, which is called after these subpackets are
@@ -1816,6 +1830,8 @@ make_keysig_packet (ctrl_t ctrl,
     {
       /* Hash the subkey binding/backsig/revocation.  */
       hash_public_key (md, subpk);
+      if ((subpk->pubkey_usage & PUBKEY_USAGE_RENC))
+        signhints |= SIGNHINT_ADSK;
     }
   else if (sigclass != 0x1F && sigclass != 0x20)
     {
@@ -1835,7 +1851,7 @@ make_keysig_packet (ctrl_t ctrl,
     sig->expiredate = sig->timestamp + duration;
   sig->sig_class = sigclass;
 
-  build_sig_subpkt_from_sig (sig, pksk);
+  build_sig_subpkt_from_sig (sig, pksk, signhints);
   mk_notation_policy_etc (ctrl, sig, pk, pksk);
 
   /* Crucial that the call to mksubpkt comes LAST before the calls
@@ -1959,6 +1975,12 @@ update_keysig_packet (ctrl_t ctrl,
       }
   }
 
+  /* Detect an ADSK key binding signature.  */
+  if ((sig->sig_class == 0x18
+       || sig->sig_class == 0x19 || sig->sig_class == 0x28)
+      && (pk->pubkey_usage & PUBKEY_USAGE_RENC))
+    signhints |= SIGNHINT_ADSK;
+
   /* Note that already expired sigs will remain expired (with a
    * duration of 1) since build-packet.c:build_sig_subpkt_from_sig
    * detects this case. */
@@ -1967,7 +1989,7 @@ update_keysig_packet (ctrl_t ctrl,
    * automagically lower any sig expiration dates to correctly
    * correspond to the differences in the timestamps (i.e. the
    * duration will shrink).  */
-  build_sig_subpkt_from_sig (sig, pksk);
+  build_sig_subpkt_from_sig (sig, pksk, signhints);
 
   if (mksubpkt)
     rc = (*mksubpkt)(sig, opaque);
