@@ -139,6 +139,7 @@ enum cmd_and_opt_values {
   oAssumeArmor,
   oAssumeBase64,
   oAssumeBinary,
+  oInputSizeHint,
 
   oBase64,
   oNoArmor,
@@ -326,6 +327,7 @@ static gpgrt_opt_t opts[] = {
                 N_("assume input is in base-64 format")),
   ARGPARSE_s_n (oAssumeBinary, "assume-binary",
                 N_("assume input is in binary format")),
+  ARGPARSE_s_s (oInputSizeHint, "input-size-hint", "@"),
 
 
   ARGPARSE_header ("Output", N_("Options controlling the output")),
@@ -802,7 +804,7 @@ set_debug (void)
 
   /* minip12.c may be used outside of GnuPG, thus we don't have the
    * opt structure over there.  */
-  p12_set_verbosity (opt.verbose);
+  p12_set_verbosity (opt.verbose, opt.debug);
 }
 
 
@@ -1188,6 +1190,10 @@ main ( int argc, char **argv)
           ctrl.is_base64 = 0;
           break;
 
+        case oInputSizeHint:
+          ctrl.input_size_hint = string_to_u64 (pargs.r.ret_str);
+          break;
+
         case oDisableCRLChecks:
           opt.no_crl_check = 1;
           break;
